@@ -31,6 +31,7 @@ local parser = nil;
 
 local config_mt = { __index = function (t, _) return rawget(t, "*"); end};
 local config = setmetatable({ ["*"] = { } }, config_mt);
+local files = {};
 
 -- When host not found, use global
 local host_mt = { __index = function(_, k) return config["*"][k] end }
@@ -98,6 +99,10 @@ function _M.load(filename, config_format)
 	end
 end
 
+function _M.files()
+	return files;
+end
+
 -- Built-in Lua parser
 do
 	local pcall = _G.pcall;
@@ -155,6 +160,11 @@ do
 			set(config_table, name or "*", "defined", true);
 			return function (config_options)
 				rawset(env, "__currenthost", "*"); -- Return to global scope
+				if type(config_options) == "string" then
+					error(format("VirtualHost entries do not accept a module name (module '%s' provided for host '%s')", config_options, name), 2);
+				elseif type(config_options) ~= "table" then
+					error("Invalid syntax following VirtualHost, expected options but received a "..type(config_options), 2);
+				end
 				for option_name, option_value in pairs(config_options) do
 					set(config_table, name or "*", option_name, option_value);
 				end
@@ -253,6 +263,8 @@ do
 			return nil, err;
 		end
 
+		t_insert(files, config_file);
+
 		return true, warnings;
 	end
 
