@@ -156,9 +156,9 @@ end
 local function request_ack(session, reason)
 	local queue = session.outgoing_stanza_queue;
 	session.log("debug", "Sending <r> (inside timer, before send) from %s - #queue=%d", reason, queue:count_unacked());
+	session.awaiting_ack = true;
 	(session.sends2s or session.send)(st.stanza("r", { xmlns = session.smacks }))
 	if session.destroyed then return end -- sending something can trigger destruction
-	session.awaiting_ack = true;
 	-- expected_h could be lower than this expression e.g. more stanzas added to the queue meanwhile)
 	session.last_requested_h = queue:count_acked() + queue:count_unacked();
 	session.log("debug", "Sending <r> (inside timer, after send) from %s - #queue=%d", reason, queue:count_unacked());
@@ -308,7 +308,8 @@ function handle_enable(session, stanza, xmlns_sm)
 	local resume_max;
 	local resume_token;
 	local resume = stanza.attr.resume;
-	if resume == "true" or resume == "1" then
+	if (resume == "true" or resume == "1") and session.username then
+		-- resumption on s2s is not currently supported
 		resume_token = new_id();
 		session_registry[jid.join(session.username, session.host, resume_token)] = session;
 		session.resumption_token = resume_token;
@@ -499,6 +500,7 @@ module:hook("pre-resource-unbind", function (event)
 		session.conn = nil;
 		conn:close();
 	end
+	session.log("debug", "Session going into hibernation (not being destroyed)")
 	module:fire_event("smacks-hibernation-start", { origin = session; queue = session.outgoing_stanza_queue:table() });
 	return true; -- Postpone destruction for now
 end);
