@@ -59,7 +59,7 @@ local function getUserElement(xml)
 			end
 		end
 	end
-	module:log("warn", "Unable to find user element");
+	module:log("warn", "Unable to find user element in %s", xml and xml:top_tag() or "nothing");
 end
 local function createOuterXml(user, host)
 	return st.stanza("server-data", {xmlns='urn:xmpp:pie:0'})
@@ -72,7 +72,7 @@ local function hex_to_base64(s)
 end
 
 local function base64_to_hex(s)
-	return base64.encode(hex.decode(s));
+	return hex.encode(base64.decode(s));
 end
 
 local handlers = {};
@@ -279,6 +279,7 @@ handlers.roster = {
 };
 
 -- PEP node configuration/etc. (not items)
+local xmlns_pubsub = "http://jabber.org/protocol/pubsub";
 local xmlns_pubsub_owner = "http://jabber.org/protocol/pubsub#owner";
 local lib_pubsub = module:require "pubsub";
 handlers.pep = {
@@ -299,6 +300,16 @@ handlers.pep = {
 			]]
 		};
 		local owner_el = user_el:get_child("pubsub", xmlns_pubsub_owner);
+		if not owner_el then
+			local pubsub_el = user_el:get_child("pubsub", xmlns_pubsub);
+			if not pubsub_el then
+				return nil;
+			end
+			for node_el in pubsub_el:childtags("items") do
+				nodes[node_el.attr.node] = true; -- relies on COMPAT behavior in mod_pep
+			end
+			return nodes;
+		end
 		for node_el in owner_el:childtags() do
 			local node_name = node_el.attr.node;
 			local node = nodes[node_name];
@@ -393,7 +404,6 @@ handlers.pep = {
 };
 
 -- PEP items
-local xmlns_pubsub = "http://jabber.org/protocol/pubsub";
 handlers.pep_ = {
 	_stores = function (self, xml) --luacheck: ignore 212/self
 		local store_names = set.new();
